@@ -2,11 +2,11 @@
 //! # cargo_auto_lib
 //!
 //! **Library crate for common tasks when building rust projects. Intended for use with cargo-auto: automation tasks written in Rust language.**  
-//! ***[repository](https://github.com/LucianoBestia/cargo_auto_lib); version: 0.7.15  date: 2021-08-19 authors: Luciano Bestia***  
+//! ***[repository](https://github.com/LucianoBestia/cargo_auto_lib); version: 0.7.19  date: 2021-08-27 authors: Luciano Bestia***  
 //!
-//! [![Lines in Rust code](https://img.shields.io/badge/Lines_in_Rust-1085-green.svg)](https://github.com/LucianoBestia/cargo_auto_lib/)
-//! [![Lines in Doc comments](https://img.shields.io/badge/Lines_in_Doc_comments-415-blue.svg)](https://github.com/LucianoBestia/cargo_auto_lib/)
-//! [![Lines in Comments](https://img.shields.io/badge/Lines_in_comments-119-purple.svg)](https://github.com/LucianoBestia/cargo_auto_lib/)
+//! [![Lines in Rust code](https://img.shields.io/badge/Lines_in_Rust-1001-green.svg)](https://github.com/LucianoBestia/cargo_auto_lib/)
+//! [![Lines in Doc comments](https://img.shields.io/badge/Lines_in_Doc_comments-380-blue.svg)](https://github.com/LucianoBestia/cargo_auto_lib/)
+//! [![Lines in Comments](https://img.shields.io/badge/Lines_in_comments-118-purple.svg)](https://github.com/LucianoBestia/cargo_auto_lib/)
 //! [![Lines in examples](https://img.shields.io/badge/Lines_in_examples-0-yellow.svg)](https://github.com/LucianoBestia/cargo_auto_lib/)
 //! [![Lines in tests](https://img.shields.io/badge/Lines_in_tests-11-orange.svg)](https://github.com/LucianoBestia/cargo_auto_lib/)
 //!
@@ -58,26 +58,10 @@
 //!
 //! ![auto_cargo_toml_to_md](https://github.com/LucianoBestia/cargo_auto_lib/raw/main/images/auto_cargo_toml_to_md.png "auto_cargo_toml_to_md")
 //!
-//! ## Functions
+//! ## based on simple functions
 //!
-//! All the functions have extensive hep/docs to describe how they work.  
+//! All the functions have extensive help/docs to describe how they work.  
 //! It is nice when you use a code editor with IntelliSense like VSCode.  
-//! Here is a list of some of them:  
-//!
-//! - `auto_cargo_toml_to_md()` - includes data from Cargo.toml to README.md files: version, authors,...
-//! - `auto_delete_old_js_snippets()` - deletes old js snippets when working with wasm-pack
-//! - `auto_lines_of_code()` - inserts shield badges with lines_of_code into README.rs
-//! - `auto_md_to_doc_comments()` - Finds rs files with markers and include segments from md files
-//! - `auto_semver_increment_minor()` - increments semver version minor part
-//! - `auto_semver_increment_patch()` - increments semver version patch part
-//! - `auto_version_from_date()` - new version as date is written toCargo.toml and service_worker.js
-//! - `package_authors_string_without_emails()` - Cargo.toml package authors as string without emails
-//! - `package_description()` - Cargo.toml package repository
-//! - `package_name()` - Cargo.toml package name
-//! - `package_repository()` - Cargo.toml package repository
-//! - `package_version()` - Cargo.toml package version
-//! - `run_shell_command()` - run one shell command
-//! - `run_shell_commands()` - run shell commands from a vector of strings.
 //!
 //! ## Caveats
 //!
@@ -98,9 +82,13 @@
 //! `///`  
 //! and it's done ! Great !
 //!
+//! ## TODO
+//!
+//! Automate badges for crates.io, doc.rs, lib.rs, license, crev review. Check if they exist and create badges.
+//!
 //! ## cargo crev reviews and advisory
 //!
-//! We leave in times of danger with `supply chain attacks`.  
+//! We leave in times of danger with [supply chain attacks](https://en.wikipedia.org/wiki/Supply_chain_attack).  
 //! It is recommended to always use [cargo-crev](https://github.com/crev-dev/cargo-crev)  
 //! to verify the trustworthiness of each of your dependencies.  
 //! Please, spread this info.  
@@ -111,7 +99,9 @@
 //!
 //! My open-source projects are free and free as a beer (MIT license).  
 //! I just love programming.  
-//! But I need also to drink. If you find my projects and tutorials helpful, please buy me a beer or two donating on my [paypal](https://www.paypal.com/paypalme/LucianoBestia). You know the price of a beer in your local bar ;-)  
+//! But I need also to drink. If you find my projects and tutorials helpful,  
+//! please buy me a beer or two donating on my [paypal](https://www.paypal.com/paypalme/LucianoBestia).  
+//! You know the price of a beer in your local bar ;-)  
 //! So I can drink a free beer for your health :-)  
 //! [Na zdravje](https://translate.google.com/?hl=en&sl=sl&tl=en&text=Na%20zdravje&op=translate) !
 //!
@@ -120,26 +110,21 @@
 // region: mod, extern and use statements
 mod auto_cargo_toml_mod;
 mod auto_cargo_toml_to_md_mod;
+mod auto_check_micro_xml_mod;
 mod auto_delete_old_js_snippets_mod;
-mod auto_github_mod;
 mod auto_helper_functions_mod;
 mod auto_lines_of_code_mod;
 mod auto_md_to_doc_comments_mod;
 mod auto_semver_mod;
+mod auto_semver_or_date_mod;
 mod auto_version_from_date_mod;
 pub mod utils_mod;
 
 // reexport functions for callers of the library
-pub use auto_cargo_toml_mod::github_owner;
-pub use auto_cargo_toml_mod::package_authors_string_without_emails;
-pub use auto_cargo_toml_mod::package_description;
-pub use auto_cargo_toml_mod::package_name;
-pub use auto_cargo_toml_mod::package_repository;
-pub use auto_cargo_toml_mod::package_version;
+pub use auto_cargo_toml_mod::CargoToml;
 pub use auto_cargo_toml_to_md_mod::auto_cargo_toml_to_md;
+pub use auto_check_micro_xml_mod::auto_check_micro_xml;
 pub use auto_delete_old_js_snippets_mod::auto_delete_old_js_snippets;
-pub use auto_github_mod::github_create_new_release;
-pub use auto_github_mod::github_upload_asset_to_release;
 pub use auto_helper_functions_mod::completion_return_one_or_more_sub_commands;
 pub use auto_helper_functions_mod::exit_if_not_run_in_rust_project_root_directory;
 pub use auto_helper_functions_mod::run_shell_command;
@@ -155,4 +140,7 @@ pub use auto_lines_of_code_mod::auto_lines_of_code;
 pub use auto_md_to_doc_comments_mod::auto_md_to_doc_comments;
 pub use auto_semver_mod::auto_semver_increment_minor;
 pub use auto_semver_mod::auto_semver_increment_patch;
+pub use auto_semver_or_date_mod::auto_version_increment_semver_or_date;
+pub use auto_semver_or_date_mod::auto_version_increment_semver_or_date_forced;
 pub use auto_version_from_date_mod::auto_version_from_date;
+pub use auto_version_from_date_mod::auto_version_from_date_forced;
